@@ -1,16 +1,23 @@
-require './spec/spec_helper.rb'
+require './spec/spec_helper'
 
 RSpec.describe Mysql2::Statement do
-  before :each do
+  before(:example) do
     @client = new_client(encoding: "utf8")
   end
 
+  let(:performance_schema_enabled) do
+    performance_schema = @client.query "SHOW VARIABLES LIKE 'performance_schema'"
+    performance_schema.any? { |x| x['Value'] == 'ON' }
+  end
+
   def stmt_count
     # Use the performance schema in MySQL 5.7 and above
-    @client.query("SELECT COUNT(1) AS count FROM performance_schema.prepared_statements_instances").first['count'].to_i
-  rescue Mysql2::Error
-    # Fall back to the global prepapred statement counter
-    @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
+    if performance_schema_enabled
+      @client.query("SELECT COUNT(1) AS count FROM performance_schema.prepared_statements_instances").first['count'].to_i
+    else
+      # Fall back to the global prepapred statement counter
+      @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
+    end
   end
 
   it "should create a statement" do
@@ -211,7 +218,7 @@ RSpec.describe Mysql2::Statement do
   end
 
   context "utf8_db" do
-    before(:each) do
+    before(:example) do
       @client.query("DROP DATABASE IF EXISTS test_mysql2_stmt_utf8")
       @client.query("CREATE DATABASE test_mysql2_stmt_utf8")
       @client.query("USE test_mysql2_stmt_utf8")
@@ -219,7 +226,7 @@ RSpec.describe Mysql2::Statement do
       @client.query("INSERT INTO テーブル (整数, 文字列) VALUES (1, 'イチ'), (2, '弐'), (3, 'さん')")
     end
 
-    after(:each) do
+    after(:example) do
       @client.query("DROP DATABASE test_mysql2_stmt_utf8")
     end
 
@@ -270,7 +277,7 @@ RSpec.describe Mysql2::Statement do
   end
 
   context "#each" do
-    # note: The current impl. of prepared statement requires results to be cached on #execute except for streaming queries
+    # NOTE: The current impl. of prepared statement requires results to be cached on #execute except for streaming queries
     #       The drawback of this is that args of Result#each is ignored...
 
     it "should yield rows as hash's" do
@@ -313,7 +320,7 @@ RSpec.describe Mysql2::Statement do
       result = @client.prepare("SELECT 1 UNION SELECT 2").execute(stream: true, cache_rows: false)
       expect do
         result.each {}
-        result.each {}
+        result.each {} # rubocop:disable Style/CombinableLoops
       end.to raise_exception(Mysql2::Error)
     end
   end
@@ -475,8 +482,13 @@ RSpec.describe Mysql2::Statement do
     end
 
     it "should raise an error given an invalid DATETIME" do
-      expect { @client.query("SELECT CAST('1972-00-27 00:00:00' AS DATETIME) as bad_datetime").each }.to \
-        raise_error(Mysql2::Error, "Invalid date in field 'bad_datetime': 1972-00-27 00:00:00")
+      if @client.info[:version] < "8.0"
+        expect { @client.query("SELECT CAST('1972-00-27 00:00:00' AS DATETIME) as bad_datetime").each }.to \
+          raise_error(Mysql2::Error, "Invalid date in field 'bad_datetime': 1972-00-27 00:00:00")
+      else
+        expect(@client.query("SELECT CAST('1972-00-27 00:00:00' AS DATETIME) as bad_datetime").to_a.first).to \
+          eql("bad_datetime" => nil)
+      end
     end
 
     context "string encoding for ENUM values" do
@@ -561,17 +573,17 @@ RSpec.describe Mysql2::Statement do
     end
 
     {
-      'char_test' => 'CHAR',
-      'varchar_test' => 'VARCHAR',
-      'varbinary_test' => 'VARBINARY',
-      'tiny_blob_test' => 'TINYBLOB',
-      'tiny_text_test' => 'TINYTEXT',
-      'blob_test' => 'BLOB',
-      'text_test' => 'TEXT',
+      'char_test'        => 'CHAR',
+      'varchar_test'     => 'VARCHAR',
+      'varbinary_test'   => 'VARBINARY',
+      'tiny_blob_test'   => 'TINYBLOB',
+      'tiny_text_test'   => 'TINYTEXT',
+      'blob_test'        => 'BLOB',
+      'text_test'        => 'TEXT',
       'medium_blob_test' => 'MEDIUMBLOB',
       'medium_text_test' => 'MEDIUMTEXT',
-      'long_blob_test' => 'LONGBLOB',
-      'long_text_test' => 'LONGTEXT',
+      'long_blob_test'   => 'LONGBLOB',
+      'long_text_test'   => 'LONGTEXT',
     }.each do |field, type|
       it "should return a String for #{type}" do
         expect(test_result[field]).to be_an_instance_of(String)
@@ -627,12 +639,12 @@ RSpec.describe Mysql2::Statement do
   end
 
   context 'last_id' do
-    before(:each) do
+    before(:example) do
       @client.query 'USE test'
       @client.query 'CREATE TABLE IF NOT EXISTS lastIdTest (`id` BIGINT NOT NULL AUTO_INCREMENT, blah INT(11), PRIMARY KEY (`id`))'
     end
 
-    after(:each) do
+    after(:example) do
       @client.query 'DROP TABLE lastIdTest'
     end
 
@@ -655,12 +667,12 @@ RSpec.describe Mysql2::Statement do
   end
 
   context 'affected_rows' do
-    before :each do
+    before(:example) do
       @client.query 'USE test'
       @client.query 'CREATE TABLE IF NOT EXISTS lastIdTest (`id` BIGINT NOT NULL AUTO_INCREMENT, blah INT(11), PRIMARY KEY (`id`))'
     end
 
-    after :each do
+    after(:example) do
       @client.query 'DROP TABLE lastIdTest'
     end
 
